;************************************
;*     Hicolor animator for GBC     *
;************************************
;
; Playback GBC hicolor pictures at 6fps.
;
; by Jeff Frohwein
;
; Started   15-Sept-99
; Last edit 16-Sept-99
;
;
;  This code works by loading another 2048+16 color palettes
; and another 4096 tiles while the current frame is displayed
; (i.e. double buffering). It uses not only vblank but also
; the screen lines that aren't updating color palettes to xfer
; data

PLAY_SHOOT      EQU     0
SHOW_FOO        EQU     0

SHOWREGA:       MACRO
        push    af
        ld      [Foo],a
        ld      a,0
        ld      [Foo+1],a
        pop     af
        ENDM

RGBSet:         MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

        INCLUDE "addsub1.inc"
        INCLUDE "hardware.inc"

;Version 1.2 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.5

SpriteRoutine EQU $ff80

ChangeRAMBank EQU $cfff

        SECTION "Low Ram",BSS

;Reserved for color pallete code

;        ds      $1000

SpriteAttrTab   DS      40*4
RAMBankToUse    DB
VBlankState     DB
SaveSP          DW
ROMBank         DB

        INCLUDE "bank.asm"

        SECTION "Org $0",HOME[$0]

Org00:  di
        jr      Org00

        SECTION "Org $38",HOME[$38]
Org38:  di
        jr      Org38

        SECTION "Org $40",HOME[$40]
        reti
;        jp      DoVblank

        SECTION "Org $48",HOME[$48]
        reti

        SECTION "Org $50",HOME[$50]
        reti
        SECTION "Org $58",HOME[$58]
        reti
        SECTION "Org $60",HOME[$60]
        reti

        SECTION "Org $66",HOME[$66]
Org68:  di
        jr      Org68

;DoVblank:
;        push    af
;        call    SpriteRoutine
;        pop     af
;        reti

BankCodeFirst:
        ld      [de],a
BankCodeLast:

; DMA copy sprite data to sprite ram

VBlankFirst:
        ld      a,SpriteAttrTab/256
        ld      [rDMA],a

        ld      a,40
VBlnkW: dec     a
        jr      nz,VBlnkW

        ret
VBlankLast:

        DB      "Date 990915"           ; Date stamping info for
                                        ; reference use only

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      begin

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "HICOLOR2",0,0,0,"0000",$80     ; Cart name   16bytes
 DB 0,0,0                     ; Not used
 DB CART_ROM                  ; Cart type
 DB CART_ROM_8M               ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1,1                       ; Maker ID
 DB 0                         ; Version     =0
 DB 0                         ; Complement check (important)
 DW 0                         ; Checksum (not important)

; Library Includes

        INCLUDE "memory1.asm"


        SECTION "Main Code",HOME[$200]

;VBlank state jump table

        dw      OverlayColors1
;        dw      OverlayColors2
        dw      OverlayColors3
;        dw      OverlayColors4
        dw      OverlayColors5
        dw      OverlayColors6
;        dw      OverlayColors7
        dw      OverlayColors8
;        dw      OverlayColors9
        dw      OverlayColors10
        dw      OverlayColors11
;        dw      OverlayColors12
        dw      OverlayColors13
        dw      TileDMA2
        dw      TileDMA1
;        dw      AttrDMA
        dw      StateReset

begin:
        di

        ld      sp,$ffff


        call    Initialize

        ld      a,1
        ld      [ROMBank],a

        ld      a,[RAMBankToUse]
        ld      [rSVBK],a       ; Set the first RAM bank

        jr      mainloop

VBlankDone:
;        halt

.loop:
        ld      a,[rLY]
        cp      5
        jr      nz,.loop

;        xor     a
;        ld      [rIF],a
;
;        ei

        jr      ml2

mainloop:
        halt
        halt
        halt
        halt
ml2:
        ld      a,[RAMBankToUse]
        ld      [rSVBK],a       ; Set the first RAM bank

        halt

        ld      de,rSVBK
        ld      hl,rBCPS
        call    $d000

; Set BG palette 4 to grey scale

        ld      a,$a0
        ld      [hl+],a

        halt

        ld      [hl],$ff
        ld      [hl],$7f

        ld      [hl],$ff  ;ce
        ld      [hl],$7f  ;39

        ld      [hl],$ff  ;8
        ld      [hl],$7f   ;21

        ld      [hl],$ff   ;0
        ld      [hl],$7f    ;0

        ld      a,[rLY]
        ld      [$cff0],a

;WaitForVBlank:
;        halt
;        ld      a,[rLY]
;        cp      143  ;143
;        jr      nz,WaitForVBlank

;        rept 8
;        halt
;        endr

        ld      h,2
        ld      a,[VBlankState]
        add     a
        ld      l,a

        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a

;        rept 8
;        halt
;        endr

;        di

        jp      [hl]

; Don't start palette updates until the 5th screen line

;VBlankDone:
;        jr      mainloop


Initialize:

        ld      a,10
        ld      [rRAMG],a           ; Enable cart RAM

        ld      a,1
        ld      [rROMB0],a

        call    DoubleSpeed

        call    ScreenOff

        xor     a
        ld      [rSCX],a
        ld      a,0
        ld      [rSCY],a

        xor     a
        ld      [VBlankState],a

; Install sprite DMA code

        ld      hl,VBlankFirst
        ld      de,SpriteRoutine
        ld      bc,VBlankLast-VBlankFirst
        call    mem_Copy

; Install change RAM bank code

        ld      hl,BankCodeFirst
        ld      de,ChangeRAMBank
        ld      bc,BankCodeLast-BankCodeFirst
        call    mem_Copy

        ld      a,$07
        ld      [rLCDC],a

; Fill $8000-9fff with 0 in both VRAM banks

        xor     a
        ld      [rVBK],a

        ld      hl,$8000
        ld      bc,$2000
        xor     a
        call    mem_Set

        ld      hl,SpriteTiles
        ld      de,$8020
        ld      bc,32
        call    mem_Copy

        ld      a,1
        ld      [rVBK],a

        ld      hl,$8000
        ld      bc,$2000
        xor     a
        call    mem_Set

; Fill attribute memory 1 with color palette 4, bank 1

        ld      a,1
        ld      [rVBK],a

        ld      hl,$9800
        ld      bc,$400
        ld      a,$c
        call    mem_Set

; Fill attribute memory 2 with color palette 4, bank 0

        ld      hl,$9c00
        ld      bc,$400
        ld      a,$4
        call    mem_Set

; Setup color palette grid in attribute memory

        ld      hl,$9822
        ld      a,$80
        ld      de,$10
        ld      c,e
.loopg1:
        ld      b,8
        xor     a
.loopg2:
        ld      [hl+],a
        ld      [hl+],a
        inc     a
        dec     b
        jr      nz,.loopg2

        add     hl,de
        dec     c
        jr      nz,.loopg1

        ld      hl,$9c22
        ld      a,$80
        ld      de,$10
        ld      c,e
.loopg3:
        ld      b,8
        ld      a,8
.loopg4:
        ld      [hl+],a
        ld      [hl+],a
        inc     a
        dec     b
        jr      nz,.loopg4

        add     hl,de
        dec     c
        jr      nz,.loopg3

; Setup tile pattern grid in tile map memory

        xor     a
        ld      [rVBK],a

        ld      hl,$9822
        call    SetupTileGrid
        ld      hl,$9c22
        call    SetupTileGrid

; Setup sprites to cover left & right borders of screen

        ld      hl,$fe00 ;SpriteAttrTab
        ld      a,19
        ld      b,10

sprlp:  ld      [hl+],a
        ld      [hl],8
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],16
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],152
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        ld      [hl+],a
        ld      [hl],160
        inc     l
        ld      [hl],2
        inc     l
        ld      [hl],0
        inc     l

        add     16
        dec     b
        jr      nz,sprlp

        ld      hl,rOCPS
        ld      [hl],$80

        inc     l

; Set all sprite palettes to white

        ld      b,32
.sprcl:
        ld      [hl],$ff
        ld      [hl],$7f
        dec     b
        jr      nz,.sprcl

; Copy tileset to VRAM & install color table

        ld      hl,$4000
        call    InstallTileSet

        ld      a,1
        ld      [RAMBankToUse],a
        ret


InstallTileSet:
        di
        xor     a
        ld      [rVBK],a

; Copy tile set to memory

        ld      de,$8800
        ld      bc,4096
        call    mem_Copy

; Install 1st color table code (banks 1-3)

        ld      a,1
        ld      [RAMBankToUse],a

        ld      hl,$5000
        call    InstallColorTable

; Install 2nd color table code (banks 4-6)

        ld      a,4
        ld      [RAMBankToUse],a

        ld      hl,$5000
        call    InstallColorTable

        ld      a,2 ;3
        ld      [rIE],a

        ld      a,8
        ld      [rSTAT],a

        xor     a
        ld      [rIF],a

        call    ScreenOn

.loop:  ld      a,[rLY]
        cp      $91
        jr      nz,.loop

        ei
        halt

        ret

SetupTileGrid:
 if 1
; Vertical array (modern)

        ld      a,$80
        ld      de,$10
        ld      c,e
.loopp1:
        ld      b,e
        push    af
.loopp2:
        ld      [hl+],a
        add     $10
        dec     b
        jr      nz,.loopp2

        pop     af
        inc     a

        add     hl,de
        dec     c
        jr      nz,.loopp1
 else
; Horizontal array (old)

        ld      a,$80
        ld      de,$10
        ld      c,e
.loopp1:
        ld      b,e
.loopp2:
        ld      [hl+],a
        inc     a
        dec     b
        jr      nz,.loopp2

        add     hl,de
        dec     c
        jr      nz,.loopp1
 endc
        ret

; Install large color table
; Entry: DE = pntr to color data
;        [BankToUse] = RAM bank start

InstallColorTable:
 if 1
        ld      d,h
        ld      e,l

        ld      hl,$d000

        ld      a,[RAMBankToUse]
        ld      [rSVBK],a       ; Set the first RAM bank

        ld      c,49             ;129
        call    .loop

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      a,[RAMBankToUse]
        inc     a
        ld      [hl],a          ; 2
        inc     hl

        ld      [hl],$c3        ; JP ChangeRAMBank
        inc     hl
        ld      [hl],(ChangeRAMBank%256)
        inc     hl
        ld      [hl],(ChangeRAMBank/256)

        ld      a,[RAMBankToUse]
        inc     a
        ld      [rSVBK],a       ; Set the second RAM bank

        ld      hl,$d000

        ld      c,50
        call    .loop

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      a,[RAMBankToUse]
        inc     a
        inc     a
        ld      [hl],a          ;3
        inc     hl

        ld      [hl],$c3        ; JP ChangeRAMBank
        inc     hl
        ld      [hl],(ChangeRAMBank%256)
        inc     hl
        ld      [hl],(ChangeRAMBank/256)

        ld      a,[RAMBankToUse]
        inc     a
        inc     a
        ld      [rSVBK],a       ; Set the third RAM bank

        ld      hl,$d000

        ld      c,30
        call    .loop

        ld      [hl],$c9        ; RET
        ret
 else
        ld      d,h
        ld      e,l

        ld      hl,$a0a0

        ld      c,129           ;129
        call    .loop

        ld      [hl],$c9        ; RET
        ret
 endc

.loop:  push    bc

        ld      [hl],$3e        ; LD A,??
        inc     hl
        ld      a,c
        inc     a
        rrca
        rrca
        rrca
        and     $20
        or      $80
        ld      [hl+],a

        ld      [hl],$22        ; LD [HL+],A
        inc     hl

        ld      [hl],$76        ; HALT
        inc     hl

        ld      b,16
        call    .loop2          ; LD [HL],?? x 32

        ld      [hl],$2d        ; DEC L
        inc     hl

        pop     bc
        dec     c
        jr      nz,.loop

        ret

.loop2:
        push    bc

        ld      a,[de]
        inc     de
        ld      c,a
        ld      a,[de]
        inc     de
        ld      b,a

        ld      a,$36
        ld      [hl+],a
        ld      [hl],c
        inc     hl
        ld      [hl+],a
        ld      [hl],b
        inc     hl
        pop     bc
        dec     b
        jr      nz,.loop2

        ret

StateReset:
        xor     a
        ld      [VBlankState],a

        ld      hl,ROMBank
        inc     [hl]
        ld      a,[hl]
        cp      5+4
        jr      c,.ok
        ld      a,1
.ok:
        ld      [$2000],a
        ld      [hl],a

OverlayColors1:
        ld      [SaveSP],sp

        di
        ld      sp,$5000

        ld      hl,$d000

        ld      a,[RAMBankToUse]
        xor     5
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

;OverlayColors2:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(1*320)

        ld      hl,$d000+(1*690)

        ld      a,[RAMBankToUse]
        xor     5
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors3:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(2*320)

        ld      hl,$d000+(2*690)

        ld      a,[RAMBankToUse]
        xor     5
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

;OverlayColors4:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(3*320)

        ld      hl,$d000+(3*690)

        ld      a,[RAMBankToUse]
        xor     5
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors5:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(4*320)

        ld      hl,$d000+(4*690)

        ld      a,[RAMBankToUse]
        xor     5
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors6:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(5*320)-32

        ld      hl,$d000+(0*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

;OverlayColors7:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(6*320)-32

        ld      hl,$d000+(1*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors8:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(7*320)-32

        ld      hl,$d000+(2*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

;OverlayColors9:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(8*320)-32

        ld      hl,$d000+(3*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors10:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(9*320)-32

        ld      hl,$d000+(4*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors11:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(10*320)-32

        ld      hl,$d000+(0*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

;OverlayColors12:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(11*320)-32

        ld      hl,$d000+(1*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

OverlayColors13:
        ld      [SaveSP],sp

        di
        ld      sp,$5000+(12*320)-32

        ld      hl,$d000+(2*690)

        ld      a,[RAMBankToUse]
        xor     5
        inc     a
        inc     a
        ld      [rSVBK],a       ; Set the other RAM banks
        jp      OverlayCommon

TileDMA1:
        rept 8
        halt
        endr

        ld      a,[rVBK]
        xor     1
        ld      [rVBK],a

        ld      a,$48
        ld      [rHDMA1],a               ; High Address value of source
        ld      a,$00
        ld      [rHDMA2],a               ; Low Address value of source

        ld      a,$90
        ld      [rHDMA3],a               ; High Address value of destination
        ld      a,$00
        ld      [rHDMA4],a               ; Low Address value of destination


        ld      a,128-1           ;127 tiles
        ld      [rHDMA5],a               ; (n-1) lines to be transferred

        ld      a,[rVBK]
        xor     1
        ld      [rVBK],a

;        ld      hl,VBlankState
;        inc     [hl]

;        jp      VBlankDone

;AttrDMA:
;        rept 8
;        halt
;        endr

        ld      a,[rLCDC]
        xor     8
        ld      [rLCDC],a

        ld      a,[RAMBankToUse]
        xor     5
        ld      [RAMBankToUse],a

        ld      a,[rVBK]
        xor     1
        ld      [rVBK],a

        ld      hl,VBlankState
        inc     [hl]

        jp      VBlankDone

TileDMA2:
        rept 8
        halt
        endr

        ld      a,[rVBK]
        xor     1
        ld      [rVBK],a

        ld      a,$40
        ld      [rHDMA1],a               ; High Address value of source
        ld      a,$00
        ld      [rHDMA2],a               ; Low Address value of source

        ld      a,$88
        ld      [rHDMA3],a               ; High Address value of destination
        ld      a,$00
        ld      [rHDMA4],a               ; Low Address value of destination

        ld      a,128-1           ;128 tiles
        ld      [rHDMA5],a               ; (n-1) lines to be transferred

        ld      a,[rVBK]
        xor     1
        ld      [rVBK],a

        ld      hl,VBlankState
        inc     [hl]

        jp      VBlankDone




OverlayCommon:
;        rept 8
;        halt
;        endr

; 223 cycles
        ld      b,20
.loop:
        inc     hl      ;2
        inc     hl      ;2 Skip 'ld a,??'
        inc     hl      ;2 Skip 'ld [hl+],a'
        inc     hl      ;2 Skip 'halt'

        rept    16

        pop     de      ;3
        ld      a,e     ;1
        inc     hl      ;2
        ld      [hl+],a ;2
        inc     hl      ;2
        ld      a,d     ;1
        ld      [hl+],a ;2

        endr

        inc     hl      ;2 skip 'dec l'

        dec     b       ;1
        jp      nz,.loop ;4

        ld      a,[SaveSP]
        ld      l,a
        ld      a,[SaveSP+1]
        ld      h,a
        ld      sp,hl

        ei

        ld      hl,VBlankState
        inc     [hl]

        jp      VBlankDone

; *** Turn screen on ***

ScreenOn:
        ld      hl,rLCDC
        set     7,[hl]
        ret

; *** Turn screen off ***

ScreenOff:
        ld      hl,rLCDC
        bit     7,[hl]          ; Is LCD already off?
        ret     z               ; yes, exit

        ld      a,[rIE]
        push    af
        res     0,a
        ld      [rIE],a         ; Disable vblank interrupt if enabled

.loop:  ld      a,[rLY]         ; Loop until in first part of vblank
        cp      145
        jr      nz,.loop

        res     7,[hl]          ; Turn the screen off

        pop     af
        ld      [rIE],a         ; Restore the state of vblank interrupt
        ret

; Set single speed CPU speed

SingleSpeed:
        ld      a,[rKEY1]
        rlca
        ret     nc
        jr      CPUToggleSpeed

; Set double speed CPU speed

DoubleSpeed:
        ld      a,[rKEY1]
        rlca
        ret     c

CPUToggleSpeed:
        di

        ld      hl,rIE
        ld      a,[hl]
        push    af

        xor     a
        ld      [hl],a         ;disable interrupts
        ld      [rIF],a

        ld      a,$30
        ld      [rP1],a

        ld      a,1
        ld      [rKEY1],a

        stop

        pop     af
        ld      [hl],a

        ei
        ret

SpriteTiles:
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
        db      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

        SECTION "Data",HOME[$3000]
fill00:
        rept    1024
        db      $00
        endr


        SECTION "car pic 1",DATA,BANK[1]
        incbin  "car//car1.til"
        incbin  "car//car1.pal"

        SECTION "car pic 2",DATA,BANK[2]
        incbin  "car//car2.til"
        incbin  "car//car2.pal"

        SECTION "car pic 3",DATA,BANK[3]
        incbin  "car//car3.til"
        incbin  "car//car3.pal"

        SECTION "car pic 4",DATA,BANK[4]
        incbin  "car//car4.til"
        incbin  "car//car4.pal"

        SECTION "car pic 5",DATA,BANK[5]
        incbin  "car//car5.til"
        incbin  "car//car5.pal"

        SECTION "car pic 6",DATA,BANK[6]
        incbin  "car//car6.til"
        incbin  "car//car6.pal"

        SECTION "car pic 7",DATA,BANK[7]
        incbin  "car//car7.til"
        incbin  "car//car7.pal"

        SECTION "car pic 8",DATA,BANK[8]
        incbin  "car//car8.til"
        incbin  "car//car8.pal"
